<?php
/**
 * @file       processImg.php
 *
 * PHP Class to resize and preprocess images in various ways
 *
 * @package		Blog - http://www.feenders.de
 * @copyright	Copyright (C) 2015 Computer - Daten - Netze : Feenders. All rights reserved.
 * @license  	GNU/GPL, see LICENSE
 * @created     10.03.15 - 17:06
 * @version     1.0
 * @author		Dirk Hoeschen (hoeschen@feenders.de)
 */

class processImg {

    // Jpeg Image Quality between 0 and 100 (no compression)
    public $quality= 85;

    // Watermak Image - Optional with complete path - must be a semitransparent png
    public $watermark = "watermark.png";

    // Transparency for watermark between 0 an 100;
    public $watermark_alpha = 10;

    // Background color in rgb values for the passepartout 40 = dark gray
    public $bgcolor = array(40, 40, 40);

    /**
     * Create - Create a new JPG-image from PNG oder JPG at destination
     * Optionally grayscale or add watermark
     *
     * Mode options:
     * scale = resize and keep ascpect
     * crop = crop image to the exact with and height
     * fit = fit image into a passepartout
     *
     * @param string $source - Name with path of the source file
     * @param string $destination  - Name with path of the destination file
     * @param int $size_w - Destination width
     * @param int $size_h - Destination height
     * @param string $mode (scale, crop, fit)
     * @param boolean $grayscale
     * @param boolean $watermark
     * @return boolean
     */
    public function create($source,$destination, $size_w=200, $size_h=200, $mode="scale",$grayscale=false,$watermark=false) {
		try {
			if (file_exists($source)) {
				if ($imageinfo = getimagesize($source)) {
                switch($imageinfo[2]) {
                    case IMAGETYPE_JPEG:
                        $src_img = imagecreatefromjpeg($source);
                        break;
                    case IMAGETYPE_PNG:
                        $src_img = imagecreatefrompng($source);
                        break;
                    case IMAGETYPE_GIF:
                        $src_img = imagecreatefromgif($source);
                        break;
					default:
						throw new Exception("Image must be of type JPG, PNG or GIF."); 	
					}	
				} else {
					throw new Exception("Can not get imageinfo from (".$source."). Is it a image?"); 	
				}	
			} else {
				throw new Exception("Can not open (".$source."). File does not exist or is not readable."); 	
			}	
		} catch (Exception $e) {
			error_log("ProcessImg: ".$e->getMessage(),0);
			return false;
		}

        if (isset($src_img)) {
            $src_width = $imageinfo[0];
            $src_height = $imageinfo[1];
            if($src_width>=$src_height) { // Aspect x>y
                $new_w = $size_w;
                $new_h = abs(($size_w/$src_width)*$src_height);
                if ($new_h>=$size_h) {
                    $new_h = $size_h;
                    $new_w = abs(($size_h/$src_height)*$src_width);
                }
            } else {
                $new_h = $size_h;
                $new_w = abs(($size_h/$src_height)*$src_width);
                if ($new_w>=$size_w) {
                    $new_w = $size_w;
                    $new_h = abs(($size_w/$src_width)*$src_height);
                }
            }
            switch ($mode) {
                case "crop" : // Crop to target size
                    $dst_img = imagecreatetruecolor($size_w,$size_h);
                    $a_new = ($size_w/$size_h);
                    $a_old = ($src_width/$src_height);
                    if($a_new>=$a_old) { // crop height
                        $sx=0;
                        $src_height = (int)($src_width/$a_new);
                        $sy = (int)(($imageinfo[1]-$src_height)/2);
                    } else { // crop width
                        $sy=0;
                        $src_width = (int)($src_height/($size_h/$size_w));
                        $sx = (int)(($imageinfo[0]-$src_width)/2);
                    }
                    imagecopyresampled($dst_img,$src_img,0,0,$sx,$sy,$size_w,$size_h,$src_width,$src_height);
                    break;
                case "fit" : // Fit into target size
                    $dst_img = imagecreatetruecolor($size_w,$size_h);
                    $color = imagecolorallocate($dst_img, $this->bgcolor[0], $this->bgcolor[1], $this->bgcolor[2]);
                    $a_new = ($size_w/$size_h);
                    $a_old = ($src_width/$src_height);
                    if($a_old>=$a_new) { // fit width
                        $dx = 0;
                        $new_w = (int) $size_w;
                        $new_h = (int)($imageinfo[1]*($new_w/$imageinfo[0]));
                        $dy = (int)(($size_h-$new_h)/2);
                    } else { // fit height
                        $dy = 0;
                        $new_h = (int) $size_h;
                        $new_w = (int)($imageinfo[0]*($new_h/$imageinfo[1]));
                        $dx = (int)(($size_w-$new_w)/2);
                    }
                    imagefill($dst_img, 0, 0, $color);
                    imagecopyresampled($dst_img,$src_img,$dx,$dy,0,0,$new_w,$new_h,$imageinfo[0],$imageinfo[1]);
                    break;
                case "scale" : // Scale (keep aspect)
                default:
                    if ($src_width<$new_w) {
                        $new_h = $src_height;
                        $new_w = $src_width;
                    }
                    $dst_img = imagecreatetruecolor($new_w,$new_h);
                    imagecopyresampled($dst_img,$src_img,0,0,0,0,$new_w,$new_h,$imageinfo[0],$imageinfo[1]);
                    break;
            }

            // get real dimensions
            $new_w = imagesx( $dst_img );
            $new_h = imagesy( $dst_img );

			// grayscale image
            if ($grayscale===true) {
                $bwimage= imagecreate($new_w,$new_h);
                $palette = array();
                //Creates the 256 color palette
                for ($c=0;$c<256;$c++){ $palette[$c] = imagecolorallocate($bwimage,$c,$c,$c);}
                //Reads the origonal colors pixel by pixel
                for ($y=0;$y<$new_h;$y++) {
                    for ($x=0;$x<$new_w;$x++) {
                        $rgb = imagecolorat($dst_img,$x,$y);
                        $r = ($rgb >> 16) & 0xFF; $g = ($rgb >> 8) & 0xFF; $b = $rgb & 0xFF;
                        //This is where we actually use yiq to modify our rbg values, and then convert them to our grayscale palette
                        $gs = (($r*0.299)+($g*0.587)+($b*0.114));
                        imagesetpixel($bwimage,$x,$y,$palette[$gs]);
                    }
                }
                $dst_img = $bwimage;
            }

			// watermark image
            if ($watermark===true) {
                if ($rWatermark = imagecreatefrompng($this->watermark)) {
					$this->imagecopymergeAlpha($dst_img, $rWatermark, 0,0,0,0, $new_w,$new_h,((int)$this->watermark_alpha%100));	
					imagedestroy($rWatermark);
                }
            }
            
            try {
				if (!imagejpeg($dst_img, $destination, ((int)$this->quality%100))) {
					throw new Exception("Can not create destination (".$dst_img.") Check path and acces rights"); 
				}				
			} catch (Exception $e) {
				error_log("ProcessImg: ".$e->getMessage(),0);
				return false;
			}
                        
            imagedestroy($dst_img);
            return true;
        } 
		error_log("ProcessImg: Something went wrong",0);
        return false;
    }

    /**
     * A fix to get a function like imagecopymerge WITH ALPHA SUPPORT
     * Nessecary to attach watermark
     *
     * @param $dst_img
     * @param $src_img
     * @param $dst_x
     * @param $dst_y
     * @param $src_x
     * @param $src_y
     * @param $w
     * @param $h
     * @param $pct
     * @return bool
     */
    protected function imagecopymergeAlpha(&$dst_img, $src_img, $dst_x, $dst_y, $src_x, $src_y, $w, $h,$pct=20) {
        $pct /= 100;
        // Get image width and height
        // Turn alpha blending off
        imagealphablending( $src_img, false );
        // Find the most opaque pixel in the image (the one with the smallest alpha value)
        $minalpha = 127;
        for( $y = 0; $y < $h; $y++ ){
            for( $x = 0; $x < $w; $x++ ) {
                $alpha = ( imagecolorat( $src_img, $x, $y ) >> 24 ) & 0xFF;
                if( $alpha < $minalpha ){
                    $minalpha = $alpha;
                }
            }
        }
        //loop through image pixels and modify alpha for each
        for( $y = 0; $y < $h; $y++ ){
            for( $x = 0; $x < $w; $x++ ){
                //get current alpha value (represents the TANSPARENCY!)
                $colorxy = imagecolorat( $src_img, $x, $y );
                $alpha = ( $colorxy >> 24 ) & 0xFF;
                //calculate new alpha
                if( $minalpha !== 127 ){
                    $alpha = 127 + 127 * $pct * ( $alpha - 127 ) / ( 127 - $minalpha );
                } else {
                    $alpha += 127 * $pct;
                }
                //get the color index with new alpha
                $alphacolorxy = imagecolorallocatealpha( $src_img, ( $colorxy >> 16 ) & 0xFF, ( $colorxy >> 8 ) & 0xFF, $colorxy & 0xFF, $alpha );
                //set pixel with the new color + opacity
                if( !imagesetpixel( $src_img, $x, $y, $alphacolorxy ) ){
                    return false;
                }
            }
        }
        // The image copy
        imagecopy($dst_img, $src_img, $dst_x, $dst_y, $src_x, $src_y, $w, $h);
    }

} 
